"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeWatchService = void 0;
const path_1 = require("path");
const fs_1 = require("fs");
const events_1 = require("events");
const utils_1 = require("@file-services/utils");
const { stat } = fs_1.promises;
class NodeWatchService {
    /**
     * Construct a new Node file system watch service
     */
    constructor(options) {
        /** user's subsribed global listeners */
        this.globalListeners = new Set();
        /** all watched paths (including files inside watched directories) */
        this.watchedPaths = new utils_1.SetMultiMap();
        /** path to actual FSWatcher instance opened for it */
        this.fsWatchers = new Map();
        /** path to its pending event (debounced watch event) */
        this.pendingEvents = new Map();
        this.options = { persistent: true, debounceWait: 200, ...options };
    }
    async watchPath(path, listener) {
        if (listener) {
            this.watchedPaths.add(path, listener);
        }
        await this.ensureFsWatcher(path);
    }
    async unwatchPath(path, listener) {
        if (listener) {
            this.watchedPaths.delete(path, listener);
        }
        else {
            this.watchedPaths.deleteKey(path);
        }
        if (!this.watchedPaths.hasKey(path)) {
            const fsWatcher = this.fsWatchers.get(path);
            if (fsWatcher) {
                fsWatcher.close();
                this.fsWatchers.delete(path);
                await (0, events_1.once)(fsWatcher, 'close');
            }
        }
        const pendingEvent = this.pendingEvents.get(path);
        if (pendingEvent) {
            clearTimeout(pendingEvent.timerId);
            this.pendingEvents.delete(path);
        }
    }
    async unwatchAllPaths() {
        for (const watcher of this.fsWatchers.values()) {
            watcher.close();
        }
        for (const { timerId } of this.pendingEvents.values()) {
            clearTimeout(timerId);
        }
        const watcherCloseEvents = Array.from(this.fsWatchers.values(), (watcher) => (0, events_1.once)(watcher, 'close'));
        this.pendingEvents.clear();
        this.fsWatchers.clear();
        this.watchedPaths.clear();
        await Promise.all(watcherCloseEvents);
    }
    addGlobalListener(listener) {
        this.globalListeners.add(listener);
    }
    removeGlobalListener(listener) {
        this.globalListeners.delete(listener);
    }
    clearGlobalListeners() {
        this.globalListeners.clear();
    }
    /**
     * Debounces watch events while retaining whether one of
     * them was a 'rename' event
     */
    onPathEvent(eventType, eventPath) {
        const pendingEvent = this.pendingEvents.get(eventPath);
        const timerId = setTimeout(() => {
            this.emitEvent(eventPath).catch((e) => this.onWatchError(e, eventPath));
        }, this.options.debounceWait);
        if (pendingEvent) {
            clearTimeout(pendingEvent.timerId);
            pendingEvent.renamed = pendingEvent.renamed || eventType === 'rename';
            pendingEvent.timerId = timerId;
        }
        else {
            this.pendingEvents.set(eventPath, { renamed: eventType === 'rename', timerId });
        }
    }
    async emitEvent(path) {
        const pendingEvent = this.pendingEvents.get(path);
        if (!pendingEvent) {
            return;
        }
        this.pendingEvents.delete(path);
        const stats = await this.statSafe(path);
        if (pendingEvent.renamed) {
            // if one of the bounced events was a rename, make sure to unwatch,
            // as the underlying native inode is now different, and our watcher
            // is not receiving events for the new one
            const existingWatcher = this.fsWatchers.get(path);
            if (existingWatcher) {
                existingWatcher.close();
                this.fsWatchers.delete(path);
                // rewatch if path points to a new inode
                if (stats) {
                    await this.ensureFsWatcher(path, stats);
                }
            }
        }
        const watchEvent = { path, stats };
        // inform global listeners
        for (const listener of this.globalListeners) {
            listener(watchEvent);
        }
        // inform path listeners
        const listeners = this.watchedPaths.get(path);
        if (listeners) {
            for (const listener of listeners) {
                listener({ path, stats });
            }
        }
    }
    async ensureFsWatcher(path, stats) {
        if (this.fsWatchers.has(path)) {
            return;
        }
        // accepting the optional stats saves us getting the stats ourselves
        const pathStats = stats || (await this.statSafe(path));
        if (!pathStats) {
            throw new Error(`cannot watch non-existing path: ${path}`);
        }
        // open fsWatcher
        const watchOptions = { persistent: this.options.persistent };
        if (pathStats.isFile()) {
            const fileWatcher = (0, fs_1.watch)(path, watchOptions, (type) => this.onPathEvent(type, path));
            fileWatcher.once('error', (e) => {
                this.onWatchError(e, path);
            });
            this.fsWatchers.set(path, fileWatcher);
        }
        else if (pathStats.isDirectory()) {
            const directoryWatcher = (0, fs_1.watch)(path, watchOptions, (type, fileName) => {
                this.onDirectoryEvent(type, path, fileName).catch;
            });
            directoryWatcher.once('error', (e) => {
                this.onWatchError(e, path);
            });
            this.fsWatchers.set(path, directoryWatcher);
        }
        else {
            throw new Error(`${path} does not point to a file or a directory`);
        }
    }
    onWatchError(_e, path) {
        this.onPathEvent('rename', path);
        const watcher = this.fsWatchers.get(path);
        if (watcher) {
            watcher.close();
        }
    }
    async onDirectoryEvent(eventType, directoryPath, fileName) {
        // we must stats the directory, as the raw event gives us no indication
        // whether an inner file or the directory itself was removed.
        // Upon removal of the directory itself, the fileName parameter is just the directory name,
        // which can also be interpreted as an inner file with that name being removed.
        const directoryStats = await this.statSafe(directoryPath);
        this.onPathEvent(eventType, directoryStats ? (0, path_1.join)(directoryPath, fileName) : directoryPath);
    }
    async statSafe(nodePath) {
        try {
            return await stat(nodePath);
        }
        catch {
            return null;
        }
    }
}
exports.NodeWatchService = NodeWatchService;
//# sourceMappingURL=watch-service.js.map