"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLanguageServiceHost = exports.createBaseHost = void 0;
const typescript_1 = __importDefault(require("typescript"));
const statsNoThrowOptions = { throwIfNoEntry: false };
const UNIX_NEW_LINE = '\n';
const identity = (val) => val;
const toLowerCase = (val) => val.toLowerCase();
const defaultGetNewLine = typescript_1.default.sys ? () => typescript_1.default.sys.newLine : () => UNIX_NEW_LINE;
/**
 * Create an IBaseHost, which is actually three interfaces combined:
 * - `ts.ParseConfigHost` - for parsing of `tsconfig.json` files
 * - `ts.FormatDiagnosticsHost` - for formatting of `ts.Diagnostic` instances
 * - `ts.ModuleResolutionHost` - for resolution of imports using TypeScript's built-in mechanism
 *
 * @param fs the file system to use as host backend
 */
function createBaseHost(fs) {
    const { caseSensitive, statSync, readFileSync, readdirSync, fileExistsSync, directoryExistsSync, cwd, realpathSync, join, dirname, normalize, } = fs;
    function getFileSystemEntries(path) {
        const files = [];
        const directories = [];
        try {
            for (const entry of readdirSync(path, { withFileTypes: true })) {
                if (entry.isFile()) {
                    files.push(entry.name);
                }
                else if (entry.isDirectory()) {
                    directories.push(entry.name);
                }
            }
        }
        catch {
            /* */
        }
        return { files, directories };
    }
    function realpathSyncSafe(path) {
        const { stackTraceLimit } = Error;
        try {
            Error.stackTraceLimit = 0;
            return realpathSync(path);
        }
        catch (e) {
            return path;
        }
        finally {
            Error.stackTraceLimit = stackTraceLimit;
        }
    }
    return {
        readDirectory(rootDir, extensions, excludes, includes, depth) {
            return typescript_1.default.matchFiles(rootDir, extensions, excludes, includes, caseSensitive, rootDir, depth, getFileSystemEntries, realpathSyncSafe, directoryExistsSync);
        },
        getDirectories(path) {
            return getFileSystemEntries(path).directories;
        },
        fileExists: fileExistsSync,
        directoryExists: directoryExistsSync,
        readFile(filePath) {
            const { stackTraceLimit } = Error;
            try {
                Error.stackTraceLimit = 0;
                return readFileSync(filePath, 'utf8');
            }
            catch {
                return undefined;
            }
            finally {
                Error.stackTraceLimit = stackTraceLimit;
            }
        },
        getScriptVersion(filePath) {
            var _a, _b;
            return `${(_b = (_a = statSync(filePath, statsNoThrowOptions)) === null || _a === void 0 ? void 0 : _a.mtime.getTime()) !== null && _b !== void 0 ? _b : Date.now()}`;
        },
        useCaseSensitiveFileNames: caseSensitive,
        getCanonicalFileName: caseSensitive ? identity : toLowerCase,
        getCurrentDirectory: cwd,
        getNewLine: defaultGetNewLine,
        realpath: realpathSyncSafe,
        dirname,
        normalize,
        join,
    };
}
exports.createBaseHost = createBaseHost;
/**
 * Create a TypeScript `LanguageServiceHost` using provided base host, list of files, and compiler options.
 *
 * @param baseHost created using `createBaseHost()`
 * @param getScriptFileNames return a list of absolute paths to `.ts/tsx` files included in this transpilation.
 * @param getCompilationSettings returns `ts.CompilerOptions` to use when transpiling or type checking.
 * @param defaultLibsDirectory absolute path to the directory that contains TypeScript's built-in `.d.ts` files
 *                             `path.dirname(ts.getDefaultLibFilePath({}))` in node,
 *                             or custom directory with `@file-services/memory`
 * @param getCustomTransformers optional transformers to apply during transpilation
 */
function createLanguageServiceHost(baseHost, getScriptFileNames, getCompilationSettings, defaultLibsDirectory, getCustomTransformers) {
    const { readFile, join, useCaseSensitiveFileNames, getNewLine } = baseHost;
    return {
        ...baseHost,
        getCompilationSettings,
        getScriptFileNames,
        getCustomTransformers,
        getScriptSnapshot(filePath) {
            const fileContents = readFile(filePath);
            return fileContents !== undefined ? typescript_1.default.ScriptSnapshot.fromString(fileContents) : undefined;
        },
        getNewLine: () => typescript_1.default.getNewLineCharacter(getCompilationSettings(), getNewLine),
        getDefaultLibFileName: (options) => join(defaultLibsDirectory, typescript_1.default.getDefaultLibFileName(options)),
        useCaseSensitiveFileNames: () => useCaseSensitiveFileNames,
    };
}
exports.createLanguageServiceHost = createLanguageServiceHost;
//# sourceMappingURL=create-host.js.map