"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addGlobals = exports.unwrapPseudoGlobals = exports.getGlobalRules = exports.hooks = exports.diagnostics = void 0;
const feature_1 = require("./feature");
const plugable_record_1 = require("../helpers/plugable-record");
const selector_1 = require("../helpers/selector");
const diagnostics_1 = require("../diagnostics");
const dataKey = plugable_record_1.plugableRecord.key('globals');
exports.diagnostics = {
    UNSUPPORTED_MULTI_SELECTOR_IN_GLOBAL: (0, diagnostics_1.createDiagnosticReporter)('04001', 'error', () => `unsupported multi selector in :global()`),
};
// HOOKS
exports.hooks = (0, feature_1.createFeature)({
    metaInit({ meta }) {
        plugable_record_1.plugableRecord.set(meta.data, dataKey, { rules: new Map() });
    },
    analyzeSelectorNode({ context, node, topSelectorIndex, rule, originalNode }) {
        var _a, _b, _c;
        var _d, _e;
        const { rules } = plugable_record_1.plugableRecord.getUnsafe(context.meta.data, dataKey);
        if (node.type === 'selector' || node.type === 'combinator' || node.type === 'comment') {
            return;
        }
        if (!rules.has(originalNode)) {
            rules.set(originalNode, {
                isGlobal: true,
                checkedRule: rule,
                topLevelSelectorsFlags: [],
            });
        }
        const ruleData = rules.get(originalNode);
        if (node.type === 'pseudo_class' && node.value === `global`) {
            // mark selector as global only if it isn't set
            (_a = (_d = ruleData.topLevelSelectorsFlags)[topSelectorIndex]) !== null && _a !== void 0 ? _a : (_d[topSelectorIndex] = true);
            if (node.nodes && ((_b = node.nodes) === null || _b === void 0 ? void 0 : _b.length) > 1) {
                context.diagnostics.report(exports.diagnostics.UNSUPPORTED_MULTI_SELECTOR_IN_GLOBAL(), {
                    node: rule,
                    word: (0, selector_1.stringifySelector)(node.nodes),
                });
            }
            return selector_1.walkSelector.skipNested;
        }
        else if (node.type === 'universal' || (node.type === 'type' && !(0, selector_1.isCompRoot)(node.value))) {
            // mark selector as global only if it isn't set
            (_c = (_e = ruleData.topLevelSelectorsFlags)[topSelectorIndex]) !== null && _c !== void 0 ? _c : (_e[topSelectorIndex] = true);
        }
        else {
            // mark selector as local if it has a local selector
            ruleData.topLevelSelectorsFlags[topSelectorIndex] = false;
        }
        return;
    },
    analyzeSelectorDone({ context, originalNode }) {
        const { rules } = plugable_record_1.plugableRecord.getUnsafe(context.meta.data, dataKey);
        const data = rules.get(originalNode);
        if (!data) {
            return;
        }
        // require at least one global selector in rule selectors
        if (!data.topLevelSelectorsFlags.find((isGlobal) => isGlobal)) {
            data.isGlobal = false;
            return;
        }
        // rule is global if it doesn't have any local parents
        let parent = originalNode.parent;
        while (parent) {
            const parentData = rules.get(parent);
            if (parentData) {
                // quick resolution: parent calculated first
                data.isGlobal = parentData.isGlobal;
                break;
            }
            // keep searching
            parent = parent.parent;
        }
    },
    transformInit({ context }) {
        context.meta.globals = {};
    },
    transformLastPass({ context: { meta }, ast }) {
        ast.walkRules((r) => {
            if (!r.selector.includes(`:global(`)) {
                return;
            }
            const selectorAst = (0, selector_1.parseSelectorWithCache)(r.selector, { clone: true });
            const globals = unwrapPseudoGlobals(selectorAst);
            addGlobals(meta, globals);
            r.selector = (0, selector_1.stringifySelector)(selectorAst);
        });
    },
});
// API
function getGlobalRules(meta) {
    const { rules } = plugable_record_1.plugableRecord.getUnsafe(meta.data, dataKey);
    const globalRules = [];
    for (const [rule, { isGlobal, checkedRule }] of rules) {
        if (isGlobal && checkedRule === rule && rule.type === 'rule') {
            globalRules.push(rule);
        }
    }
    return globalRules;
}
exports.getGlobalRules = getGlobalRules;
function unwrapPseudoGlobals(selectorAst) {
    const collectedGlobals = [];
    (0, selector_1.walkSelector)(selectorAst, (node) => {
        var _a;
        if (node.type === 'pseudo_class' && node.value === 'global') {
            collectedGlobals.push(node);
            if (((_a = node.nodes) === null || _a === void 0 ? void 0 : _a.length) === 1) {
                (0, selector_1.flattenFunctionalSelector)(node);
            }
            return selector_1.walkSelector.skipNested;
        }
        return;
    });
    return collectedGlobals;
}
exports.unwrapPseudoGlobals = unwrapPseudoGlobals;
function addGlobals(meta, selectorAst) {
    for (const ast of selectorAst) {
        (0, selector_1.walkSelector)(ast, (inner) => {
            if (inner.type === 'class') {
                // ToDo: consider if to move to css-class feature.
                meta.globals[inner.value] = true;
            }
        });
    }
}
exports.addGlobals = addGlobals;
//# sourceMappingURL=st-global.js.map