"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.StylableManifestPlugin = void 0;
const core_1 = require("@stylable/core");
const index_internal_1 = require("@stylable/core/dist/index-internal");
const node_1 = require("@stylable/node");
const create_metadata_stylesheet_1 = require("./create-metadata-stylesheet");
const hash_content_util_1 = require("./hash-content-util");
const path_1 = require("path");
const os_1 = require("os");
const defaultOptions = {
    package: {
        name: 'default-name',
        version: '0.0.0-default',
    },
    outputType: 'manifest',
    packageAlias: {},
    resolveNamespace: node_1.resolveNamespace,
    exposeNamespaceMapping: true,
    generateCSSVarsExports: false,
    filterComponents(resourcePath) {
        return resourcePath.endsWith('.comp.st.css');
    },
    getCompId(resourcePath) {
        return (0, path_1.basename)(resourcePath).replace(/\.comp\.st\.css$/, '');
    },
    getOutputFileName(contentHash) {
        return `stylable.manifest.${contentHash}.json`;
    },
};
function generateCssVarsNamedExports(name, meta) {
    return Object.keys(index_internal_1.STSymbol.getAllByType(meta, `cssVar`))
        .map((varName) => `${varName} as --${name}-${varName.slice(2)}`)
        .join(',');
}
class StylableManifestPlugin {
    constructor(options = {}) {
        this.options = Object.assign({}, defaultOptions, options);
    }
    apply(compiler) {
        const stylable = new core_1.Stylable({
            projectRoot: compiler.context,
            fileSystem: {
                readlinkSync: (filePath) => compiler.inputFileSystem.readlinkSync(filePath),
                statSync: (filePath) => compiler.inputFileSystem.statSync(filePath),
                readFileSync: (filePath) => compiler.inputFileSystem.readFileSync(filePath).toString(),
            },
            mode: compiler.options.mode === 'development' ? 'development' : 'production',
            resolveOptions: {
                ...compiler.options.resolve,
                extensions: [],
            },
            resolverCache: new Map(),
            resolveNamespace: this.options.resolveNamespace,
        });
        compiler.hooks.done.tap(this.constructor.name + ' stylable.initCache', () => stylable.initCache());
        let metadataList;
        compiler.hooks.compilation.tap(this.constructor.name, (compilation) => {
            compilation.hooks.optimizeModules.tap(this.constructor.name, (modules) => {
                metadataList = this.createModulesMetadata(compiler, stylable, [...modules]);
            });
            compilation.hooks.processAssets.tap(this.constructor.name, () => this.emitManifest(metadataList, compilation));
        });
    }
    emitManifest(metadataList, compilation) {
        const manifest = metadataList.reduce((manifest, { meta, compId, metadata }) => {
            const cssVars = this.options.generateCSSVarsExports
                ? generateCssVarsNamedExports(compId, meta)
                : null;
            Object.assign(manifest.stylesheetMapping, metadata.stylesheetMapping);
            Object.assign(manifest.namespaceMapping, metadata.namespaceMapping);
            manifest.componentsEntries[compId] = metadata.entry;
            manifest.componentsIndex += `:import{-st-from: ${JSON.stringify(metadata.entry)};-st-default: ${compId};${cssVars ? `-st-named:${cssVars};` : ``}} .root ${compId}{}${os_1.EOL}`;
            return manifest;
        }, {
            name: this.options.package.name,
            version: this.options.package.version,
            componentsIndex: '',
            componentsEntries: {},
            stylesheetMapping: {},
            namespaceMapping: {},
        });
        if (this.options.outputType === 'fs-manifest') {
            this.emitJSONAsset(convertToFsMetadata(manifest, this.options.packageAlias), compilation);
        }
        else {
            this.emitJSONAsset(manifest, compilation);
        }
    }
    emitJSONAsset(manifest, compilation) {
        const manifestContent = JSON.stringify(manifest);
        const contentHash = (0, hash_content_util_1.hashContent)(manifestContent, this.options.contentHashLength);
        compilation.emitAsset(this.options.getOutputFileName(contentHash), new compilation.compiler.webpack.sources.RawSource(manifestContent, false));
    }
    createModulesMetadata(compiler, stylable, modules) {
        const stylableComps = modules.filter((module) => {
            const resource = module.resource;
            return resource && this.options.filterComponents(resource);
        });
        return stylableComps.map((module) => {
            const resource = module.resource;
            const source = compiler.inputFileSystem.readFileSync(resource).toString();
            const meta = stylable.fileProcessor.processContent(source, resource);
            return {
                meta,
                compId: this.options.getCompId(resource),
                metadata: (0, create_metadata_stylesheet_1.createMetadataForStylesheet)(stylable, source, resource, this.options.exposeNamespaceMapping, meta),
            };
        });
    }
}
exports.StylableManifestPlugin = StylableManifestPlugin;
/* This supports the output of previous version of the metadata plugin */
const convertToFsMetadata = (manifest, packages) => {
    const pkg = { name: manifest.name, version: manifest.version };
    const normalizedMetadata = {
        ...pkg,
        fs: {
            [`/${manifest.name}/index.st.css`]: {
                metadata: {
                    /* naive package name to css class might need to support more characters */
                    namespace: manifest.name.replace(/[@/]/g, '_'),
                },
                content: manifest.componentsIndex,
            },
        },
        components: {},
        packages,
    };
    Object.keys(manifest.stylesheetMapping).forEach((filePath) => {
        normalizedMetadata.fs[filePath] = {
            metadata: {
                namespace: manifest.namespaceMapping[filePath],
            },
            content: manifest.stylesheetMapping[filePath],
        };
    });
    Object.keys(manifest.componentsEntries).forEach((id) => {
        const stylesheetPath = manifest.componentsEntries[id];
        const namespace = manifest.namespaceMapping[stylesheetPath];
        normalizedMetadata.components[id] = {
            id,
            stylesheetPath,
            namespace,
        };
    });
    return normalizedMetadata;
};
//# sourceMappingURL=stylable-manifest-plugin.js.map