"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.isErrorLikeObject = exports.errorToPlainObject = exports.stringifyErrorStack = exports.UnreachableCaseError = exports.ErrorWithCode = exports.getErrorCode = exports.toError = void 0;
const objects_1 = require("./objects");
/**
 * Convert any kind of value to an error instance. Unless the value is already
 * an error instance, it's stringified and used as the error message.
 */
function toError(value) {
    return value instanceof Error ? value : new Error(value === undefined ? undefined : String(value));
}
exports.toError = toError;
/**
 * Returns error.code property if the error object has it, otherwise returns undefined.
 */
function getErrorCode(error) {
    return typeof error.code === 'string' ? error.code : undefined;
}
exports.getErrorCode = getErrorCode;
/**
 */
class ErrorWithCode extends Error {
    constructor(message, options) {
        const { code, ...nativeErrorOptions } = options !== null && options !== void 0 ? options : {};
        super(message, nativeErrorOptions);
        this.code = code;
    }
}
exports.ErrorWithCode = ErrorWithCode;
/**
 * Allows the type checker to detect non-exhaustive switch statements.
 * @example
 * ```ts
 * declare const align: 'left' | 'right' | 'middle';
 * switch (align) {
 *     case 'left': return 1;
 *     case 'right': return 2;
 *     // type error since 'middle' is not handled
 *     default: throw new UnreachableCaseError(align);
 * }
 * ```
 */
class UnreachableCaseError extends Error {
    constructor(switchValue) {
        super(`Unreachable switch case: ${JSON.stringify(switchValue)}`);
    }
}
exports.UnreachableCaseError = UnreachableCaseError;
function stringifyErrorStack(error) {
    if (isErrorLikeObject(error)) {
        // in v8, e.stack includes "<name>: <message>"
        // in other js runtimes, it only includes the stack
        const { name, message, stack = '' } = error;
        const namePlusMessage = `${name}: ${message}`;
        return stack.startsWith(namePlusMessage) ? stack : `${namePlusMessage}\n${stack}`;
    }
    return String(error);
}
exports.stringifyErrorStack = stringifyErrorStack;
function errorToPlainObject(error) {
    return {
        ...error,
        message: error.message,
        name: error.name,
        stack: error.stack,
    };
}
exports.errorToPlainObject = errorToPlainObject;
function isErrorLikeObject(error) {
    return ((0, objects_1.isPlainObject)(error) &&
        typeof error.name === 'string' &&
        typeof error.message === 'string');
}
exports.isErrorLikeObject = isErrorLikeObject;
//# sourceMappingURL=errors.js.map