"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.remap = exports.getIn = exports.defaults = exports.reverseObject = exports.newMacrotask = exports.awaitRecord = exports.reportError = exports.isPlainObject = exports.mapKeys = exports.mapValues = exports.mapObject = exports.pick = exports.exclude = void 0;
const promise_assist_1 = require("promise-assist");
const chain_1 = require("./chain");
function exclude(...excluded) {
    return function (t) {
        for (const item of excluded) {
            if (item === t) {
                return false;
            }
        }
        return true;
    };
}
exports.exclude = exclude;
/**
 * returns an object composed of the picked object properties
 * @example
 * ```ts
 * pick({ a: 1, b: 2 }, ['a']) // => { a: 1 }
 * ```
 */
function pick(record, keys) {
    const subset = {};
    for (const key of keys) {
        subset[key] = record[key];
    }
    return subset;
}
exports.pick = pick;
/**
 * Maps key value pairs of a plain object
 */
function mapObject(obj, mapping) {
    return Object.fromEntries(Object.entries(obj).map(mapping));
}
exports.mapObject = mapObject;
/**
 * Maps values of a plain object
 */
function mapValues(obj, mapping) {
    return Object.fromEntries(Object.entries(obj).map(([k, v]) => [k, mapping(v, k)]));
}
exports.mapValues = mapValues;
/**
 * Maps values of a plain object
 */
function mapKeys(obj, mapping) {
    return Object.fromEntries(Object.entries(obj).map(([k, v]) => [mapping(k, v), v]));
}
exports.mapKeys = mapKeys;
/**
 * Checks that value is a POJO
 */
function isPlainObject(value) {
    return value !== null && typeof value === 'object' && Object.getPrototypeOf(value) === Object.prototype;
}
exports.isPlainObject = isPlainObject;
/**
 * Logs an error
 */
function reportError(ex) {
    // eslint-disable-next-line no-console, @typescript-eslint/no-unsafe-member-access, @typescript-eslint/no-unsafe-call, @typescript-eslint/no-unsafe-return
    console.error(ex);
}
exports.reportError = reportError;
/**
 * Awaits a record of promises, and returns a record of their results.
 */
async function awaitRecord(obj) {
    const out = {};
    for (const [key, promise] of Object.entries(obj)) {
        out[key] = await promise;
    }
    return out;
}
exports.awaitRecord = awaitRecord;
const newMacrotask = () => (0, promise_assist_1.sleep)(0);
exports.newMacrotask = newMacrotask;
/**
 * Reverses keys-values of an object, ignoring falsy values.
 * First takes on value collisions.
 * @returns a new object with the values as keys and the keys as values
 * @example
 * ```ts
 * reverseObject({ a: 'y', b: 'z'}) // => { y: 'a', z: 'b' }
 * ```
 */
function reverseObject(obj) {
    const reversedObject = {};
    for (const [key, value] of Object.entries(obj)) {
        if (value) {
            reversedObject[value] || (reversedObject[value] = key);
        }
    }
    return reversedObject;
}
exports.reverseObject = reverseObject;
/**
 * Returns an object where missing keys and values/keys
 * that satisfy shouldUseDefault
 * to the value in shouldUseDefault.
 *
 * @example
 * ```ts
 * defaults({}, {a:0}) // => {a:0}
 * defaults({a:1}, {a:0}) // => {a:1}
 * defaults({a:{}}, {a:{b:1}}) // => {a:{b:1}}
 * ```
 * by default, any undefined value will be replaced
 * @param deep - [true] perform a deep comparison
 * @example
 * ```ts
 * defaults({a:{}}, {a:{b:1}}, false) // => {a:{}}
 * ```
 * @param shouldUseDefault - value/key for which shouldUseDefault
 * returns true will be taken from defaultValues,
 * ignoring source.
 * k is provided as a dot separated path
 * @example
 * ```ts
 * defaults({a:{b:1}}, {a:{b:2}}, true, (_,k)=>k==='a.b') // => {a:{b:2}}
 * defaults({a:1}, {a:2}, true, v=>v===1) // => {a:2}
 * ```
 * @returns a new object with merged source and defaultValues
 */
function defaults(_source, _defaultValues, deep = true, shouldUseDefault = (v, _key) => v === undefined) {
    const parseObj = (src, dft, parentKey = '') => {
        if (isPlainObject(src)) {
            const result = {};
            for (const [key, value] of Object.entries(src)) {
                const fullKey = (parentKey ? parentKey + '.' : '') + key;
                const _default = isPlainObject(dft) ? dft[key] : undefined;
                if (shouldUseDefault(value, fullKey)) {
                    result[key] = _default;
                }
                else {
                    result[key] = deep ? parseObj(value, _default, fullKey) : value;
                }
            }
            if (isPlainObject(dft)) {
                for (const [key, _default] of Object.entries(dft)) {
                    if (!(key in src)) {
                        // eslint-disable-next-line @typescript-eslint/no-unsafe-assignment
                        result[key] = _default;
                    }
                }
            }
            return result;
        }
        else {
            // @ts-expect-error non object values should only affect type when misused
            return shouldUseDefault(src) ? dft : src;
        }
    };
    return parseObj(_source, _defaultValues);
}
exports.defaults = defaults;
/**
 * @param obj - The object to query
 * @param path - The path of the property to get.
 * @returns The value at `path` of `object` id exists, `undefined` otherwise
 * @example
 * ```ts
 * getIn({ a: { b: 'c' } }, ['a', 'b']) // => c
 * ```
 */
function getIn(obj, path) {
    // eslint-disable-next-line @typescript-eslint/no-unsafe-return
    return path.reduce((value, key) => value === null || value === void 0 ? void 0 : value[key], obj);
}
exports.getIn = getIn;
const DELETE = Symbol();
/**
 * remaps keys of obj based on rename map object,
 * @param rename - key:key name in obj, value: new name (string) OR remap.DELETE
 * @returns
 */
const remap = (obj, rename) => Object.fromEntries((0, chain_1.chain)(Object.entries(obj))
    // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
    .filter(([key]) => rename[key] !== exports.remap.DELETE)
    .map(([key, value]) => 
// eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
[rename[key] || key, value]).iterable);
exports.remap = remap;
// @ts-expect-error setting the DELETE const
exports.remap.DELETE = DELETE;
//# sourceMappingURL=objects.js.map