"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.same = void 0;
const iterables_1 = require("./iterables");
const objects_1 = require("./objects");
const types_1 = require("./types");
/**
 * Deep comparison of two objects

 * @example
 * ```ts
 * same({a:1, b:2},{b:2, a:1}) // => true
 * same(new Map([['a',1],['b',2]]), new Map([['b',2],['a',1]])) // => true
 * same(new Map([[{a:1},{b:1}]]]), new Map([[{a:1},{b:1}]]])) // => true
 * same([0,1], [1,0]) // => false
 * same([0,1], [1,0], true) // => true
 * ```
 * @remarks
 * items are "the same" if:
 *
 * - a === b (for anything other than iterables & POJO)

 * - a and b are POJO with same entries (order ignored, {@link same} used to compare values)
 *
 * - a and b are Maps with same entries (order ignored,  {@link same} used to compare keys & values)
 *
 * - a and b are Sets with same values (order ignored,  {@link same} used to compare values)
 *
 * - a and b are iterable (that are not Set or Map) with the same values, order checked if unordered=false (default)
 *
 * @param unordered - [false] Note: relevant only in array like iterables. objects, sets and maps are *never checked for order* of entries
 */
function same(a, b, unordered = false) {
    if (a === b)
        return true;
    if (!(0, iterables_1.isIterable)(a) || !(0, iterables_1.isIterable)(b)) {
        if ((0, objects_1.isPlainObject)(a) && (0, objects_1.isPlainObject)(b)) {
            return sameIgnoreOrder(Object.entries(a), Object.entries(b), unordered);
        }
        return a === b;
    }
    return unordered || ((0, types_1.isMap)(a) && (0, types_1.isMap)(b)) || ((0, types_1.isSet)(a) && (0, types_1.isSet)(b))
        ? sameIgnoreOrder(a, b, unordered)
        : sameInOrder(a, b, unordered);
}
exports.same = same;
function sameInOrder(a, b, unordered) {
    if (!(0, iterables_1.isIterable)(a) || !(0, iterables_1.isIterable)(b)) {
        if ((0, objects_1.isPlainObject)(a) && (0, objects_1.isPlainObject)(b)) {
            return sameIgnoreOrder(Object.entries(a), Object.entries(b), unordered);
        }
        return a === b;
    }
    const itA = a[Symbol.iterator]();
    const itB = b[Symbol.iterator]();
    for (let [aa, bb] = [itA.next(), itB.next()];; [aa, bb] = [itA.next(), itB.next()]) {
        if (!same(aa.value, bb.value, unordered)) {
            return false;
        }
        if (aa.done !== bb.done) {
            return false;
        }
        if (aa.done) {
            return true;
        }
    }
}
function sameIgnoreOrder(a, b, unordered) {
    const [aa, bb] = [(0, iterables_1.histogram)(a), (0, iterables_1.histogram)(b)];
    if ((0, iterables_1.size)(aa) !== (0, iterables_1.size)(bb)) {
        return false;
    }
    for (const [key, count] of aa) {
        let wasFound = false;
        for (const [bkey, bcount] of bb) {
            if (same(key, bkey, unordered)) {
                wasFound = true;
                if (bcount !== count) {
                    return false;
                }
                else {
                    break;
                }
            }
        }
        if (!wasFound) {
            return false;
        }
    }
    return true;
}
//# sourceMappingURL=same.js.map