import { isPlainObject } from './objects';
/**
 * Convert any kind of value to an error instance. Unless the value is already
 * an error instance, it's stringified and used as the error message.
 */
export function toError(value) {
    return value instanceof Error ? value : new Error(value === undefined ? undefined : String(value));
}
/**
 * Returns error.code property if the error object has it, otherwise returns undefined.
 */
export function getErrorCode(error) {
    return typeof error.code === 'string' ? error.code : undefined;
}
/**
 */
export class ErrorWithCode extends Error {
    constructor(message, options) {
        const { code, ...nativeErrorOptions } = options !== null && options !== void 0 ? options : {};
        super(message, nativeErrorOptions);
        this.code = code;
    }
}
/**
 * Allows the type checker to detect non-exhaustive switch statements.
 * @example
 * ```ts
 * declare const align: 'left' | 'right' | 'middle';
 * switch (align) {
 *     case 'left': return 1;
 *     case 'right': return 2;
 *     // type error since 'middle' is not handled
 *     default: throw new UnreachableCaseError(align);
 * }
 * ```
 */
export class UnreachableCaseError extends Error {
    constructor(switchValue) {
        super(`Unreachable switch case: ${JSON.stringify(switchValue)}`);
    }
}
export function stringifyErrorStack(error) {
    if (isErrorLikeObject(error)) {
        // in v8, e.stack includes "<name>: <message>"
        // in other js runtimes, it only includes the stack
        const { name, message, stack = '' } = error;
        const namePlusMessage = `${name}: ${message}`;
        return stack.startsWith(namePlusMessage) ? stack : `${namePlusMessage}\n${stack}`;
    }
    return String(error);
}
export function errorToPlainObject(error) {
    return {
        ...error,
        message: error.message,
        name: error.name,
        stack: error.stack,
    };
}
export function isErrorLikeObject(error) {
    return (isPlainObject(error) &&
        typeof error.name === 'string' &&
        typeof error.message === 'string');
}
//# sourceMappingURL=errors.js.map