"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OrderedSlot = exports.OrderedRegistry = void 0;
const symbols_1 = require("../symbols");
const input_1 = require("./input");
function getItemOrder(sortingOrder, item) {
    const index = sortingOrder.indexOf(item);
    return index < 0 ? Infinity : index;
}
function compareAny(a, b) {
    return a < b ? -1 : a > b ? 1 : 0;
}
function composeCompare(f, g) {
    return (a, b) => {
        const r = f(a, b);
        return r || g(a, b);
    };
}
const { hasOwnProperty } = Object.prototype;
function mkCompare(params) {
    let func = (_a, _b) => 0;
    // eslint-disable-next-line @typescript-eslint/no-for-in-array
    for (const p in params) {
        if (hasOwnProperty.call(params, p)) {
            const [key, mode] = params[p];
            if (mode === true) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                func = composeCompare(func, (a, b) => compareAny(b[key], a[key]));
            }
            else if (mode === false) {
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                func = composeCompare(func, (a, b) => compareAny(a[key], b[key]));
            }
            else {
                func = composeCompare(func, 
                // eslint-disable-next-line @typescript-eslint/no-unsafe-member-access
                (a, b) => getItemOrder(mode, a[key]) - getItemOrder(mode, b[key]));
            }
        }
    }
    return func;
}
class OrderedRegistry {
    constructor() {
        this.items = [];
        this.callbacks = new Set();
        this.compareFunction = (_a, _b) => 0;
    }
    register(item) {
        this.items.push(item);
        this.sortItems();
        for (const callback of this.callbacks) {
            callback(item);
        }
    }
    [Symbol.iterator]() {
        return this.items[Symbol.iterator]();
    }
    subscribe(cb) {
        this.callbacks.add(cb);
    }
    unSubscribe(cb) {
        this.callbacks.delete(cb);
    }
    stream(cb) {
        for (const item of this) {
            cb(item);
        }
        this.subscribe(cb);
        return () => this.unSubscribe(cb);
    }
    setItems(items) {
        this.items = items;
        this.sortItems();
        return this;
    }
    setSortingOrder(order) {
        this.compareFunction = mkCompare(order);
        this.sortItems();
        return this;
    }
    sortItems() {
        this.items.sort(this.compareFunction);
    }
}
exports.OrderedRegistry = OrderedRegistry;
class OrderedSlot extends input_1.FeatureInput {
    static withType() {
        return {
            defineEntity(env) {
                return new OrderedSlot(env, env);
            },
        };
    }
    [symbols_1.CREATE_RUNTIME]() {
        return new OrderedRegistry();
    }
}
exports.OrderedSlot = OrderedSlot;
//# sourceMappingURL=ordered-slot.js.map