"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.canAccessWindow = exports.getTopWindow = exports.FeatureLoadersRegistry = exports.runEngineApp = exports.getFeaturesDeep = exports.run = void 0;
const communication_feature_1 = __importDefault(require("./communication.feature"));
const runtime_engine_1 = require("./runtime-engine");
const helpers_1 = require("./helpers");
const promise_assist_1 = require("promise-assist");
function run({ entryFeature, topLevelConfig = [], env, runOptions, }) {
    return new runtime_engine_1.RuntimeEngine(env, topLevelConfig, runOptions).run(entryFeature);
}
exports.run = run;
const getFeaturesDeep = (feature) => (0, helpers_1.flattenTree)(feature, (f) => f.dependencies);
exports.getFeaturesDeep = getFeaturesDeep;
function runEngineApp({ config = [], options, env, publicPath, features = [], resolvedContexts = {}, }) {
    const engine = new runtime_engine_1.RuntimeEngine(env, [communication_feature_1.default.use({ config: { resolvedContexts, publicPath } }), ...config], options);
    const runningPromise = engine.run(features);
    return {
        engine,
        async dispose() {
            await runningPromise;
            for (const feature of features) {
                await engine.dispose(feature);
            }
        },
    };
}
exports.runEngineApp = runEngineApp;
class FeatureLoadersRegistry {
    constructor(featureMapping = new Map(), resolvedContexts = {}) {
        this.featureMapping = featureMapping;
        this.resolvedContexts = resolvedContexts;
        this.pendingLoaderRequests = new Map();
        this.loadedFeatures = new Set();
    }
    register(name, featureLoader) {
        var _a;
        this.featureMapping.set(name, featureLoader);
        (_a = this.pendingLoaderRequests.get(name)) === null || _a === void 0 ? void 0 : _a.resolve(featureLoader);
        this.pendingLoaderRequests.delete(name);
    }
    get(featureName) {
        const featureLoader = this.featureMapping.get(featureName);
        if (featureLoader) {
            return featureLoader;
        }
        let loaderPromise = this.pendingLoaderRequests.get(featureName);
        if (!loaderPromise) {
            loaderPromise = (0, promise_assist_1.deferred)();
            this.pendingLoaderRequests.set(featureName, loaderPromise);
        }
        return loaderPromise.promise;
    }
    getRegistered() {
        return [...this.featureMapping.keys()];
    }
    /**
     * returns all features which were actially loaded
     */
    async getLoadedFeatures(rootFeatureName, runtimeOptions = {}) {
        const loaded = [];
        const dependencies = await this.getFeatureDependencies(rootFeatureName);
        for await (const depName of dependencies.reverse()) {
            if (!this.loadedFeatures.has(depName)) {
                this.loadedFeatures.add(depName);
                const loader = this.get(depName);
                loaded.push(loader);
            }
        }
        const featureLoaders = await Promise.all(loaded);
        const allPreloadInitFunctions = [];
        for (const featureLoader of featureLoaders) {
            if (featureLoader.preload) {
                const featureInitFunctions = await featureLoader.preload(this.resolvedContexts);
                if (featureInitFunctions) {
                    allPreloadInitFunctions.push(...featureInitFunctions);
                }
            }
        }
        for (const initFunction of allPreloadInitFunctions) {
            await initFunction(runtimeOptions);
        }
        return Promise.all(featureLoaders.map(({ load }) => load(this.resolvedContexts)));
    }
    /**
     * returns all the dependencies of a feature. doesn't handle duplications
     */
    async getFeatureDependencies(featureName) {
        const dependencies = [featureName];
        const features = [featureName];
        while (features.length) {
            const { depFeatures } = await this.get(features.shift());
            for (const depFeature of depFeatures) {
                if (!dependencies.includes(depFeature)) {
                    dependencies.push(depFeature);
                    features.push(depFeature);
                }
            }
        }
        return dependencies;
    }
}
exports.FeatureLoadersRegistry = FeatureLoadersRegistry;
function getTopWindow(win) {
    while (win.parent && win.parent !== win && canAccessWindow(win.parent)) {
        win = win.parent;
    }
    return win;
}
exports.getTopWindow = getTopWindow;
function canAccessWindow(win) {
    try {
        return typeof win.location.search === 'string';
    }
    catch {
        return false;
    }
}
exports.canAccessWindow = canAccessWindow;
//# sourceMappingURL=run-engine-app.js.map