"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RuntimeEngine = void 0;
const entities_1 = require("./entities");
const symbols_1 = require("./symbols");
class RuntimeEngine {
    constructor(entryEnvironment, topLevelConfig = [], runOptions = new Map()) {
        this.entryEnvironment = entryEnvironment;
        this.runOptions = runOptions;
        this.features = new Map();
        this.running = false;
        this.topLevelConfigMap = this.createConfigMap(topLevelConfig);
        this.referencedEnvs = new Set([...entities_1.globallyProvidingEnvironments, ...(0, entities_1.orderedEnvDependencies)(entryEnvironment)]);
    }
    get(feature) {
        const runningFeature = this.features.get(feature);
        if (runningFeature) {
            return runningFeature;
        }
        else {
            throw new Error(`missing feature ${feature.id}`);
        }
    }
    async run(features) {
        if (this.running) {
            throw new Error('Engine already running!');
        }
        this.running = true;
        if (!Array.isArray(features)) {
            features = [features];
        }
        for (const feature of features) {
            this.initFeature(feature);
        }
        const runPromises = [];
        for (const feature of features) {
            runPromises.push(this.runFeature(feature));
        }
        await Promise.all(runPromises);
        return this;
    }
    initFeature(feature) {
        let instance = this.features.get(feature);
        if (!instance) {
            instance = feature[symbols_1.CREATE_RUNTIME](this);
        }
        return instance;
    }
    async runFeature(feature) {
        const featureInstance = this.features.get(feature);
        if (!featureInstance) {
            throw new Error('Could not find running feature: ' + feature.id);
        }
        await featureInstance[symbols_1.RUN](this);
    }
    async dispose(feature) {
        const runningFeature = this.features.get(feature);
        if (runningFeature) {
            await runningFeature[symbols_1.DISPOSE](this);
            this.features.delete(feature);
        }
    }
    getTopLevelConfig(featureId, configId) {
        return this.topLevelConfigMap[this.entityID(featureId, configId)] || [];
    }
    entityID(featureId, entityKey) {
        return `${featureId}.${entityKey}`;
    }
    createConfigMap(topLevelConfig) {
        const configMap = {};
        for (const [featureId, multiValue] of topLevelConfig) {
            for (const [configId, value] of Object.entries(multiValue)) {
                const key = this.entityID(featureId, configId);
                configMap[key] = configMap[key] || [];
                configMap[key].push(value);
            }
        }
        return configMap;
    }
}
exports.RuntimeEngine = RuntimeEngine;
//# sourceMappingURL=runtime-engine.js.map