/**
 * Signal is a simple event emitter for one type of event.

 * @example
 * ```ts
 * const foodArrived = new Signal<Food>();
 *
 * foodArrived.subscribe(() => {
 *   console.log('Food arrived!');
 * });
 *
 * foodArrived.notify(new Food('pizza'));
 * ```
 *
 * @example Usage in a class:
 * ```ts
 * class LoginService {
 *     public onLoginSuccess = new Signal<User>();
 *     public onLoginFailure = new Signal<Error>();
 *     public onLoginStatusChange = new Signal<Status>();
 * }
 * ```
 * @remarks
 * Use Signals a public api for emitting events.
 * Naming a signal is like naming the event the it triggers.
 * If the name sounds like a property try to add a `on` prefix or `Change/Signal` suffix.
 * All methods are bound to the Signal instance
 *
 * Notice that the Signals are public.
 * We don't need to implement specific subscriptions on the class, unless we need to expose it as a remote service.
 */
export class Signal extends Set {
    constructor() {
        super(...arguments);
        /**
         * Subscribe a notification callback
         * @param handler - Will be executed with a data arg when a notification occurs
         */
        this.subscribe = (handler) => {
            this.add(handler);
        };
        /**
         * Unsubscribe an existing callback
         */
        this.unsubscribe = (handler) => {
            this.delete(handler);
        };
        /**
         * Notify all subscribers with arg data
         */
        this.notify = (data) => {
            for (const handler of this) {
                handler(data);
            }
        };
    }
}
/**
 * Basic type safe event emitter

 * @example
 * ```ts
 * const ms = new EventEmitter<{ onChange: { id: 'onChange' }; onDelete: { id: 'onDelete' } }>();
 * ms.subscribe('onChange', (event) => {
 *    event.id; // 'onChange'
 * });
 * ms.subscribe('onDelete', (event) => {
 *   event.id; // 'onDelete'
 * });
 *
 * ms.notify('onChange', { id: 'onChange' }); // event is type safe
 * ```
 * @example <caption>payload type mismatch</caption>
 * ```ts
 * ms.notify('onChange', { id: 'onDelete' }); // ERROR!!!
 * ```
 * @example <caption>payload type mismatch</caption>
 * ```ts
 * ms.notify('onSomethingElse', { id: 'onDelete' }); // ERROR!!!
 * ```
 */
//# sourceMappingURL=signal.js.map