"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.updateLayers = exports.createBackdropParts = void 0;
const tabbable_1 = require("tabbable");
const utils_1 = require("./utils");
function createBackdropParts() {
    return {
        block: utils_1.isBrowser ? document.createElement(`zeejs-block`) : {},
        hide: utils_1.isBrowser ? document.createElement(`zeejs-hide`) : {},
    };
}
exports.createBackdropParts = createBackdropParts;
const asyncFocusChangePromise = new WeakMap();
async function updateLayers(wrapper, topLayer, { hide, block }, { asyncFocusChange, forceFocus } = {}) {
    const focusedElement = document.activeElement;
    const layers = topLayer.generateDisplayList();
    let blocking = null;
    let hiding = null;
    const layersElementsToKeep = new Set();
    // append new layers, set order and find backdrop position
    for (const [index, { element, settings }] of layers.entries()) {
        if (element) {
            layersElementsToKeep.add(element);
            element.style.zIndex = String(index);
            if (element.parentElement !== wrapper) {
                wrapper.appendChild(element);
            }
        }
        if (settings.backdrop !== `none`) {
            blocking = element;
            if (settings.backdrop === `hide`) {
                hiding = element;
            }
        }
    }
    // - remove un-needed layers/backdrop
    // - find activated layers
    const activatedLayers = [];
    const blockedIndex = blocking ? Number(blocking.style.zIndex) : 0;
    for (const element of Array.from(wrapper.children)) {
        if (element instanceof HTMLElement && layersElementsToKeep.has(element)) {
            const index = Number(element.style.zIndex);
            if (index < blockedIndex) {
                element.setAttribute(`inert`, ``);
            }
            else if (element.hasAttribute(`inert`)) {
                activatedLayers.push(element);
                element.removeAttribute(`inert`);
            }
        }
        else {
            wrapper.removeChild(element);
        }
    }
    // append backdrop if needed
    if (hiding) {
        wrapper.insertBefore(hide, hiding);
        hide.style.zIndex = hiding.style.zIndex;
    }
    if (blocking) {
        wrapper.insertBefore(block, blocking);
        block.style.zIndex = blocking.style.zIndex;
    }
    // reference active element in layer + inert check
    let elementToBlur;
    let elementToFocus;
    let prevFocusLayer = null;
    let nextFocusLayer = null;
    if (focusedElement) {
        prevFocusLayer = (0, utils_1.findContainingLayer)(focusedElement);
        if (prevFocusLayer && prevFocusLayer.hasAttribute(`inert`)) {
            elementToBlur = focusedElement;
        }
    }
    // find re-focus last input from activated layer
    if (forceFocus ||
        (!prevFocusLayer && activatedLayers.length) ||
        (prevFocusLayer &&
            document.compareDocumentPosition(prevFocusLayer) &
                document.DOCUMENT_POSITION_DISCONNECTED)) {
        // search layer with previously focused element
        for (let i = layers.length - 1; i >= 0; --i) {
            const layer = layers[i];
            if (layer.element.hasAttribute(`inert`)) {
                break;
            }
            if (layer.state.lastFocusedElement) {
                nextFocusLayer = layer.element;
                elementToFocus = layer.state.lastFocusedElement;
                break;
            }
        }
    }
    // sync/async blur/refocus
    if (elementToBlur || elementToFocus || nextFocusLayer) {
        if (asyncFocusChange) {
            let buffered = asyncFocusChangePromise.get(wrapper);
            if (buffered) {
                buffered.blur = elementToBlur;
                buffered.focus = elementToFocus;
                buffered.focusLayer = nextFocusLayer;
            }
            else {
                buffered = {
                    promise: Promise.resolve().then(() => {
                        const change = asyncFocusChangePromise.get(wrapper);
                        if (change) {
                            changeFocus(change);
                        }
                        asyncFocusChangePromise.delete(wrapper);
                    }),
                    blur: elementToBlur,
                    focus: elementToFocus,
                    focusLayer: nextFocusLayer,
                };
                asyncFocusChangePromise.set(wrapper, buffered);
            }
            return buffered.promise;
        }
        else {
            changeFocus({
                blur: elementToBlur,
                focus: elementToFocus,
                focusLayer: nextFocusLayer,
            });
        }
    }
}
exports.updateLayers = updateLayers;
function changeFocus({ blur, focus, focusLayer, }) {
    if (blur) {
        blur.blur();
    }
    if (focus && (0, tabbable_1.isFocusable)(focus)) {
        focus.focus();
    }
    else if (focusLayer && !focusLayer.hasAttribute(`inert`)) {
        // ToDo: currently find the first focusable when focused is to possible
        // solve issues:
        // 1. find closest in layer
        // 2. handle no focusable elements in layer (including layer itself)
        const focusList = (0, tabbable_1.tabbable)(focusLayer, { includeContainer: true });
        if (focusList.length) {
            focusList[0].focus();
        }
    }
}
//# sourceMappingURL=update-layers.js.map